<?php

namespace FiloBlu\StockaFix\Model\ResourceModel;


/**
 * Class ReportOutputCollection
 * @package FiloBlu\StockaFix\Model\ResourceModel
 */
class ReportOutputCollection extends \Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection implements \Magento\Framework\Api\Search\SearchResultInterface
{
    /**
     * @var int
     */
    protected $totalCount = null;

    /**
     * @var
     */
    protected $items = [];

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $request;

    /**
     * @var
     */
    protected $aggregations;

    /**
     * @var
     */
    protected $searchCriteria;

    /**
     * @var Report\CollectionFactory
     */
    protected $reportCollectionFactory;

    /**
     * @var \FiloBlu\StockaFix\Helper\ReportHelper
     */
    protected $reportHelper;

    /**
     * @var \FiloBlu\StockaFix\Model\Report
     */
    protected $report;

    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $resourceConnection;

    /**
     * ReportOutputCollection constructor.
     * @param \Magento\Framework\Data\Collection\EntityFactoryInterface $entityFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\Data\Collection\Db\FetchStrategyInterface $fetchStrategy
     * @param \Magento\Framework\Event\ManagerInterface $eventManager
     * @param Report\CollectionFactory $reportCollectionFactory
     * @param \FiloBlu\StockaFix\Helper\ReportHelper $reportHelper
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Magento\Framework\App\ResourceConnection $resourceConnection
     * @param \Magento\Framework\DB\Adapter\AdapterInterface|null $connection
     * @param \Magento\Framework\Model\ResourceModel\Db\AbstractDb|null $resource
     * @throws \Exception
     */
    public function __construct(
        \Magento\Framework\Data\Collection\EntityFactoryInterface $entityFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Data\Collection\Db\FetchStrategyInterface $fetchStrategy,
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \FiloBlu\StockaFix\Model\ResourceModel\Report\CollectionFactory $reportCollectionFactory,
        \FiloBlu\StockaFix\Helper\ReportHelper $reportHelper,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \Magento\Framework\DB\Adapter\AdapterInterface $connection = null,
        \Magento\Framework\Model\ResourceModel\Db\AbstractDb $resource = null
    )
    {

        $this->resourceConnection = $resourceConnection;
        $this->reportHelper = $reportHelper;
        $this->_eventManager = $eventManager;
        $this->_resource = $resource;
        $this->request = $request;
        $this->reportCollectionFactory = $reportCollectionFactory;
        $collection = $this->reportCollectionFactory->create();

        parent::__construct(
            $entityFactory,
            $logger,
            $fetchStrategy,
            $eventManager,
            $connection,
            $collection->getResource());
    }

    /**
     * @return string
     */
    public function getMainTable()
    {
        return $this->_mainTable;
    }

    /**
     * @param bool $printQuery
     * @param bool $logQuery
     * @return $this|\Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection
     * @throws \Zend_Db_Statement_Exception
     */
    public function load($printQuery = false, $logQuery = false)
    {
        $this->items = [];

        $offset = ($this->_pageSize * ($this->_curPage - 1));
        $page = $this->_pageSize;

        $this->totalCount = $this->_select->query()->rowCount();

        $result = $this->_select->limit($page, $offset)->query()->fetchAll(\Zend_Db::FETCH_ASSOC);

        $i = 1;
        foreach ($result as $item) {
            $t = new  \FiloBlu\StockaFix\Model\ReportOutput();

            $t->setId($i++);
            foreach ($item as $k => $v) {
                $t->setCustomAttribute($k, $v);
            }

            $this->_items[] = $t;
        }

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function setItems(array $items = null)
    {
        $this->items = $items;
    }

    /**
     * @return AggregationInterface
     */
    public function getAggregations()
    {
        return $this->aggregations;
    }

    /**
     * @param AggregationInterface $aggregations
     * @return $this
     */
    public function setAggregations($aggregations)
    {
        $this->aggregations = $aggregations;
        return $this;
    }

    /**
     * @return SearchCriteriaInterface
     */
    public function getSearchCriteria()
    {
        return $this->searchCriteria;
    }

    /**
     * @param \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
     * @return $this|\Magento\Framework\Api\Search\SearchResultInterface
     */
    public function setSearchCriteria(\Magento\Framework\Api\SearchCriteriaInterface $searchCriteria)
    {
        $this->searchCriteria = $searchCriteria;
        return $this;
    }

    /**
     * @return int
     * @throws \Zend_Db_Statement_Exception
     */
    public function getTotalCount()
    {
        if($this->totalCount === null)
        {
            $this->totalCount = $this->_select->query()->rowCount();
        }

        return $this->totalCount;
    }

    /**
     * @param int $totalCount
     * @return $this|\Magento\Framework\Api\Search\SearchResultInterface
     */
    public function setTotalCount($totalCount)
    {
        $this->totalCount = $totalCount;
        return $this;
    }

    /**
     * @return array
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getAllIds()
    {
        $idsSelect = clone $this->getSelect();
        $idsSelect->reset(\Magento\Framework\DB\Select::ORDER);
        $idsSelect->reset(\Magento\Framework\DB\Select::LIMIT_COUNT);
        $idsSelect->reset(\Magento\Framework\DB\Select::LIMIT_OFFSET);
        $idsSelect->reset(\Magento\Framework\DB\Select::COLUMNS);
        $idsSelect->columns($this->getResource()->getIdFieldName(), 'main_table');
    }

    /**
     * @return \Magento\Framework\Model\ResourceModel\Db\AbstractDb|null
     */
    public function getResource()
    {
        return $this->_resource;
    }

    /**
     * @throws \Exception
     */
    protected function _construct()
    {
        $this->items = [];
        $collection = $this->reportCollectionFactory->create();
        $this->_resource = $collection->getResource();
        $this->report = $collection
            ->addFieldToFilter('report_id', ['eq', $this->request->getParam('id')])
            ->getFirstItem();
        $this->_mainTable = $this->reportHelper->getReportSqlView($this->report);
        $this->setConnection($collection->getConnection());
        $this->setMainTable(
            $this->_mainTable);
    }

    /**
     * @param string $table
     * @return $this|\Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection
     */
    public function setMainTable($table)
    {
        $this->_mainTable = $table;
        return $this;
    }

    /**
     * @return $this|\Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection
     * @throws \Zend_Db_Statement_Exception
     */
    protected function _initInitialFieldsToSelect()
    {
        $meta = $this->reportHelper->getReportMeta($this->report);

        foreach ($meta as $attribute => $info) {
            $this->addFieldToSelect($attribute);
        }

        return $this;
    }

    /**
     * @return $this|\Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection|void
     */
    protected function _initSelect()
    {
        $this->_select = $this->reportCollectionFactory->create()->getConnection()->select();
        parent::_initSelect();
        return $this;
    }
}